
from unicodedata import name
import bpy
import bmesh
import re
import math
import statistics
import itertools
from mathutils import Vector

def execute(self, context):
    C = bpy.context
    props = C.scene.UVsProps
    props.alws_hide = self
class BoundarySeamOperator(bpy.types.Operator):
    '''Mark edge seam around linked flat surface
Wouldn't crossing between UV island'''
    '''Similar to "Select Linked Faces" with auto marking seam around selected'''
    bl_idname = "mesh.boundaryseam"
    bl_label = "Mark Around Flat"
    bl_options = {'REGISTER', 'UNDO'}

    angle: bpy.props.FloatProperty(default= 0.0174533,
                                    min= 0,
                                    description= "Set maximum angle flatten surface",
                                    unit= 'ROTATION') # type: ignore
    step: bpy.props.IntProperty(min= 0,
                                soft_max= 10,
                                max = 32,
                                description= "Grow selection similar to Select More/Less",
                                default= 0) # type: ignore

    hide: bpy.props.BoolProperty(default= False,
                                description= "Hide selected for ease navigation",
                                options= {'SKIP_SAVE'}) # type: ignore

    fc_only: bpy.props.BoolProperty(name= "",
                                default= False,
                                description= "Show edges (Hide faces only)",) # type: ignore
    dialog: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'}) # type: ignore
    dis_lu: bpy.props.BoolProperty(default= False,
                                description= "Ignore this if want to use \"Live Unwarp\" anyway",
                                options= {'SKIP_SAVE'}) # type: ignore
    hi_faces: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'}) # type: ignore
    force_auto: bpy.props.BoolProperty(name = 'Force Unwrap', # type: ignore
                          description= '''If this option showed up, it means the object tris exceeds the max Tris setting.
For performance reasons it's recommended to disable this option.
you can adjust the max Tris setting in Easeam preferences or enable this option to force unwrap anyway.''',
                          default = False,
                            options={'SKIP_SAVE'})

    def execute(self, context):

        C = bpy.context

        ms= C.active_object.data
        mesh = bmesh.from_edit_mesh(ms)
        props = C.scene.UVsProps

        # face = len(C.object.data.polygons) 
        # if face > 10000 and props.live_uv == True:
        #     # return context.window_manager.invoke_props_dialog(self)
        #     self.dialog = True

        # if C.object.data.count_selected_items()[2] == 0:
        #     self.report({"ERROR"}, "No face(s) selected")
        #     return {"CANCELLED"}


        self.face_count = len(C.object.data.polygons)
        if self.face_count >= props.max_tris:
            if not self.force_auto:
                props.unwrap_mode = 'NONE'



        live_u = C.tool_settings.use_edge_path_live_unwrap

        if props.unwrap_mode == 'LIVE_UV':
            C.tool_settings.use_edge_path_live_unwrap = True
        else:
            C.tool_settings.use_edge_path_live_unwrap = False
        if self.dis_lu == True:
            props.live_uv = False
        
        getf = [f for f in mesh.faces if f.select == True]

        bpy.ops.mesh.select_linked(delimit={'SEAM'})
        deflat = [f for f in mesh.faces if f.select == False]
        bpy.ops.mesh.select_all(action='DESELECT')
        for i in getf:
            i.select = True
        bpy.ops.mesh.faces_select_linked_flat(sharpness= self.angle)
        # defa = [f for f in mesh.faces if f.select == False]
        for i in deflat:
            i.select = False


        se_face = []
        for step in range(self.step):
                    
            edge = []
            s_edge = []
            vert = [v for v in mesh.verts if v.select == True]
            for e in mesh.edges:
                if e.select == True:
                    s_edge.append(e)
                if e.seam == True:
                    edge.append(e)

            es = []
            for v in vert:
                for e in v.link_edges:
                    if e.seam == True:
                        es.append(v)

            es = list(dict.fromkeys(es))
            for i in es:
                if i in vert:
                    vert.remove(i)

            for i in vert:
                for f in i.link_faces:
                    while f.select == False:
                        f.select = True
                        se_face.append(f)

            for e in s_edge:
                if e.seam == False:
                    for f in e.link_faces:
                        f.select = True
                        se_face.append(f)


        # Convert to edge
        getf = [f for f in mesh.faces if f.select == True]
        bpy.ops.mesh.region_to_loop()
        gete = [f for f in mesh.edges if f.select == True]

        bpy.ops.mesh.mark_seam(clear=False)
        bpy.ops.mesh.select_mode(type="FACE")

        
        for i in getf:
            i.select = True
            if props.alws_hide:
                i.hide = True
                if not self.fc_only:
                    for e in i.edges:
                        e.hide= True
                        for es in gete:
                            es.hide= False
                if self.hide:
                    for e in i.edges:
                        e.hide= False
                    for e in i.edges:
                        e.hide= False
                    i.hide = False
            
            else:
                if self.hide:
                    i.hide = True
                    if not self.fc_only:
                        for e in i.edges:
                            e.hide= True
                            for es in gete:
                                es.hide= False
        if props.unwrap_mode == 'SELECTED':
            bpy.ops.uv.unwrap()

        bmesh.update_edit_mesh(mesh= ms)
        C.tool_settings.use_edge_path_live_unwrap = live_u
            
        return {'FINISHED'}


    def draw(self, context):
        
        scene = context.scene
        props = scene.UVsProps
        layout = self.layout

        C = bpy.context
        props = C.scene.UVsProps

        if self.dialog == True and self.dis_lu == False:
            col = layout.box()
            col = col.column(align= True)
            col.label(icon= 'ERROR')
            col.label(text= "\"Use Live Unwrap\" is enabled")
            col.label(text= "Might be slow on high amount faces")
            # col.prop(props, "no_ask")
            col.prop(self, "dis_lu", text= "Disable Live Unwrap and Close", icon= 'CANCEL')

        row = layout.row(align= True)
        row.scale_y= 1.2

        if props.alws_hide:
            if self.hide == True:
                ico = 'HIDE_OFF'
                txt = 'Hide'
            else:
                ico = 'HIDE_ON'
                txt = 'Unhide'
            inv= True
        else:
            if self.hide == True:
                ico = 'HIDE_ON'
                txt = 'Unhide'
            else:
                ico = 'HIDE_OFF'
                txt = 'Hide'
            inv= False
        row.prop(self, "hide", text= txt, icon= ico, invert_checkbox= inv)


        if self.hide or props.alws_hide:
            row.prop(self, "fc_only", icon= 'SHADING_WIRE', icon_only= True)


        col = layout.column()

        col = layout.column(align= True)
        col.prop(self, "angle", text= "Angle")
        col.prop(self, "step", text= "Grow")
        
        row = layout.row(align= True)
        if self.face_count >= props.max_tris:
            row.prop(self, "force_auto")
            row.label(text = f"Faces Count: {self.face_count}")
            row = layout.row(align= True)
            row.enabled =self.force_auto
        row.prop(props, "unwrap_mode", expand = True)

class ReUnwrapOperator(bpy.types.Operator):
    '''Re-unwrap whole mesh (included hiden geometry)
But keep current selection'''
    bl_idname = "mesh.reunwrap"
    bl_label = "ReUnwrap"
    bl_options = {'REGISTER', 'UNDO'}

    sel_only: bpy.props.BoolProperty(name= "Only Selected",
                                default= False,
                                description= "Only re-unwarp selected geometry")

    inc_hide: bpy.props.BoolProperty(name= "Include hiden",
                                default= True,
                                description= "Also re-unwarp hiden geometry")
    
    exclude_pin :bpy.props.BoolProperty(name= "Keep Pinned",
                                default= True,
                                description= "Ignore Pinned UV Island")
    
    def execute(self, context):
        C= bpy.context
        scene = C.scene
        easeam_prop = scene.UVsProps
        ms = C.active_object.data
        mesh = bmesh.from_edit_mesh(ms)
        objs= C.selected_objects

        for obj in objs:
            if obj.scale != Vector((1.0, 1.0, 1.0)):
                self.report({'WARNING'}, 'Some Objects has not reset scale, Unwrap ratio may mistake.')


        verts = [e for e in mesh.verts if e.select == True]

        if self.inc_hide:
            bpy.ops.mesh.reveal()
        if not self.sel_only:
            bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.uv.unwrap(method= easeam_prop.u_method,
                        fill_holes= easeam_prop.fl_hol,
                        correct_aspect= easeam_prop.cr_aspect,
                        use_subsurf_data= easeam_prop.us_subf,
                        margin= easeam_prop.mrgn)
        
        if easeam_prop.auto_pack:
            bpy.ops.uv.pack_islands(rotate_method='AXIS_ALIGNED',
                                    margin= easeam_prop.mrgn,
                                    pin= self.exclude_pin)

        bpy.ops.mesh.select_all(action='DESELECT')
        for i in verts:
            i.select = True

        mesh.select_flush(True)
        bmesh.update_edit_mesh(mesh= ms)

        return {'FINISHED'}

    def draw(self, context):
        C= bpy.context
        scene = C.scene
        easeam_prop = scene.UVsProps

        scene = context.scene
        layout = self.layout

        row= layout.row()
        row.prop(easeam_prop, "auto_pack", text= 'Auto Pack')
        row.prop(self, "exclude_pin")
        col= layout.column()
        col.prop(easeam_prop, "mrgn")
        col.prop(easeam_prop, "u_method", expand= True)
        col= layout.row()
        col.prop(self, "inc_hide")
        col= layout.row()
        col.prop(self, "sel_only")
        

def get_loops(edge):
    e = edge[0]

    for loop in e.link_loops:
        if len(loop.vert.link_edges) == 4:
            e.select = True

            while len(loop.vert.link_edges) == 4:
                loop = loop.link_loop_prev.link_loop_radial_prev.link_loop_prev

                e_next = loop.edge
                for e2 in e_next.link_loops:
                    if not e2.edge.seam:
                        e_next.select = True
                    # else:
                    #     break
                        loop2 = e2.link_loop_prev.link_loop_radial_prev.link_loop_prev
                        loop2.edge.select = False
                # break

class LimitLoopOperator(bpy.types.Operator):
    '''Select Edge Loop and marking seam
Wouldn't crossing the edges has already marked seam'''
    bl_idname = "mesh.limitloop"
    bl_label = "LimitLoop"
    bl_options = {'REGISTER', 'UNDO'}

    force_auto: bpy.props.BoolProperty(name = 'Force Unwrap', # type: ignore
                          description= '''If this option showed up, it means the object tris exceeds the max Tris setting.
For performance reasons it's recommended to disable this option.
you can adjust the max Tris setting in Easeam preferences or enable this option to force unwrap anyway.''',
                          default = False,
                            options={'SKIP_SAVE'})
    
    def execute(self, context):
        # start_time = time.time()
        C = bpy.context
        scene = C.scene
        ms = C.active_object.data
        props = scene.UVsProps

        self.face_count = len(C.object.data.polygons)
        if self.face_count >= props.max_tris:
            if not self.force_auto:
                props.unwrap_mode = 'NONE'


        if C.object.data.count_selected_items()[1] == 0:
            self.report({"ERROR"}, "No Edge(s) selected")
            return {"CANCELLED"}

        if props.unwrap_mode == 'LIVE_UV':
            C.tool_settings.use_edge_path_live_unwrap = True
        else:
            C.tool_settings.use_edge_path_live_unwrap = False

        mesh = bmesh.from_edit_mesh(ms)
        bmesh.update_edit_mesh(mesh= ms)


        hiden_faces = [f for f in mesh.faces if f.hide == True]
        hiden_edges = [e for e in mesh.edges if e.hide == True]
        hiden_verts = [v for v in mesh.verts if v.hide == True]

        sm_edge = [e for e in mesh.edges if e.seam == True]
        init_edge = [e for e in mesh.edges if e.select == True]
        
        bpy.ops.mesh.select_all(action='DESELECT')
        edges_result = []
        for edge in init_edge:
            if edge.seam:
                continue
            bpy.ops.mesh.select_mode(type='VERT')
            edge.select = True

            bpy.ops.mesh.loop_multi_select(ring=False)
            init_loop = [e for e in mesh.edges if e.select == True]

            # return ({'FINISHED'})
            #check face
            # has_face = [e for e in mesh.faces if e.select == True]
            # if has_face:
            #     self.report({'ERROR'}, 'Adjacent edges is detected, Please execute one by one or fix the topology.')
            #     return ({'CANCELLED'})
            
            bpy.ops.mesh.select_all(action='DESELECT')
            for e in sm_edge:
                for v in e.verts:
                    v.select = True
                    
                for f in e.link_faces:
                    for e2 in f.edges:
                        if e2 not in init_loop:
                            e2.hide = True

            mesh.select_flush(True)
            bpy.ops.mesh.hide(unselected=False)
            bpy.ops.mesh.select_mode(type='EDGE')
            for e in init_edge:
                e.hide = False
                e.select = True
            bpy.ops.mesh.loop_multi_select(ring=False)

            for e in init_edge:
                e.select = True
            bpy.ops.mesh.loop_multi_select(ring=False)
            # second_loop = [e for e in mesh.edges if e.select == True]

            for e in sm_edge:
                for f in e.link_faces:
                    for e2 in f.edges:
                        if e2 not in hiden_edges:
                            e2.hide = False

            for e in mesh.edges:
                if e not in hiden_edges:
                    e.hide = False
            bpy.ops.mesh.select_more(use_face_step=False)
            more_edges = [e for e in mesh.edges if e.select == True]
            # return {'FINISHED'}
            for e in more_edges:
                e.select = False
                for f in e.link_faces:
                    for e2 in f.edges:
                        if e2 in init_loop:
                            e2.select = True

            for e in mesh.edges:
                if e not in init_loop:
                    e.select = False

            final_edges = [e for e in mesh.edges if e.select == True]
            # unhide
            for f in mesh.faces:
                if f not in hiden_faces:
                    f.hide = False
                    
            for v in mesh.verts:
                if v not in hiden_verts:
                    v.hide = False
            
            edges_result.append(final_edges)


            bpy.ops.mesh.select_all(action='DESELECT')
            bmesh.update_edit_mesh(mesh= ms)
        
        edges_result = [item for sublist in edges_result for item in sublist]
        for e in edges_result:
            e.select = True

        if props.s_loop:
            bpy.ops.mesh.mark_seam(clear= False)

        if props.unwrap_mode == 'SELECTED' and props.s_loop:
            bpy.ops.mesh.select_linked(delimit={'SEAM'})
            bpy.ops.uv.unwrap(method= props.u_method,
                            fill_holes= props.fl_hol,
                            correct_aspect= props.cr_aspect,
                            use_subsurf_data= props.us_subf,
                            margin= props.mrgn)
            bpy.ops.mesh.select_all(action='DESELECT')
            for e in final_edges:
                e.select = True

        bmesh.update_edit_mesh(mesh= ms)

        # end_time = time.time()
        # execution_time = end_time - start_time
        # print(f"Waktu eksekusi: {execution_time} detik")
        
        return {'FINISHED'}
    
    def draw(self, context):
        
        scene = context.scene
        C = bpy.context
        props = scene.UVsProps
        layout = self.layout

        row = layout.row()
        row.prop(props, "s_loop")

        row = layout.row(align= True)
        if self.face_count >= props.max_tris:
            row.prop(self, "force_auto")
            row.label(text = f"Faces Count: {self.face_count}")
            row = layout.row(align= True)
            row.enabled =self.force_auto
        row.prop(props, "unwrap_mode", expand = True)

    
class SmartSeamOperator(bpy.types.Operator):
    '''Auto unwrap also mark seam
based build-in "Smart UV Project" operator'''
    bl_idname = "mesh.smartseam"
    bl_label = "Auto Unwrap & Seam"
    bl_options = {'REGISTER', 'UNDO'}
    
    
    agl_limit: bpy.props.FloatProperty()
    dialog: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'})
    dis_lu: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'})

    def execute(self, context):
        C = bpy.context
        scene = C.scene
        easeam_prop = scene.UVsProps

        live_u= C.tool_settings.use_edge_path_live_unwrap
        C.tool_settings.use_edge_path_live_unwrap = False
        if easeam_prop.live_uv == True:
            C.tool_settings.use_edge_path_live_unwrap = True
        # if self.dis_lu == True:
        #     easeam_prop.live_uv = False

        if not easeam_prop.os_unwrap:
            bpy.ops.mesh.select_all(action='SELECT')

        # if C.object.data.count_selected_items()[2] == 0:
        #     self.report({"ERROR"}, "\"Only Selected\" is Enabled\nBut No face(s) selected")
        #     return {"CANCELLED"}

        bpy.ops.mesh.mark_seam(clear=True)
        
        bpy.ops.uv.smart_project(angle_limit= easeam_prop.angle_limit)
        bpy.ops.uv.seams_from_islands()

        C.tool_settings.use_edge_path_live_unwrap= live_u

        return {'FINISHED'}

    def draw(self, context):
        
        scene = context.scene
        props = scene.UVsProps
        layout = self.layout

        C = bpy.context
        props = C.scene.UVsProps

        row = layout.row()
        row.prop(props, "angle_limit")
        row = layout.row()
        row.prop(props, "os_unwrap")
  
class SmartSeamOperator(bpy.types.Operator):
    '''Auto unwrap also mark seam
based build-in "Smart UV Project" operator'''
    bl_idname = "mesh.smartseam"
    bl_label = "Auto Unwrap & Seam"
    bl_options = {'REGISTER', 'UNDO'}
    
    
    agl_limit: bpy.props.FloatProperty()
    dialog: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'})
    dis_lu: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'})

    def execute(self, context):
        C = bpy.context
        scene = C.scene
        easeam_prop = scene.UVsProps

        live_u= C.tool_settings.use_edge_path_live_unwrap
        C.tool_settings.use_edge_path_live_unwrap = False
        if easeam_prop.live_uv == True:
            C.tool_settings.use_edge_path_live_unwrap = True
        # if self.dis_lu == True:
        #     easeam_prop.live_uv = False

        if not easeam_prop.os_unwrap:
            bpy.ops.mesh.select_all(action='SELECT')

        # if C.object.data.count_selected_items()[2] == 0:
        #     self.report({"ERROR"}, "\"Only Selected\" is Enabled\nBut No face(s) selected")
        #     return {"CANCELLED"}

        bpy.ops.mesh.mark_seam(clear=True)
        
        bpy.ops.uv.smart_project(angle_limit= easeam_prop.angle_limit)
        bpy.ops.uv.seams_from_islands()

        C.tool_settings.use_edge_path_live_unwrap= live_u

        return {'FINISHED'}

    def draw(self, context):
        
        scene = context.scene
        props = scene.UVsProps
        layout = self.layout

        C = bpy.context
        props = C.scene.UVsProps

        row = layout.row()
        row.prop(props, "angle_limit")
        row = layout.row()
        row.prop(props, "os_unwrap")



class EaseamMarkSeamOperator(bpy.types.Operator):
    '''Mark seam on selected'''
    bl_idname = "mesh.easeammarkseam"
    bl_label = "Easeam Mark Seam"
    bl_options = {'REGISTER', 'UNDO'}

    toggle: bpy.props.BoolProperty(name= 'Toggle Mode',
                                options= {'SKIP_SAVE'},
                                default= False,)
    
    auto_u: bpy.props.BoolProperty(name= 'Auto Unwarp',
                                default= True,)
    
    force_auto: bpy.props.BoolProperty(name = 'Force Unwrap', # type: ignore
                          description= '''If this option showed up, it means the object tris exceeds the max Tris setting.
For performance reasons it's recommended to disable this option.
you can adjust the max Tris setting in Easeam preferences or enable this option to force unwrap anyway.''',
                          default = False,
                            options={'SKIP_SAVE'})
    
    def execute(self, context):
        C= bpy.context
        scene = C.scene
        props = scene.UVsProps

        ms = C.active_object.data
        mesh = bmesh.from_edit_mesh(ms)

        init_edge = [e for e in mesh.edges if e.select]

        self.face_count = len(C.object.data.polygons)
        if self.face_count >= props.max_tris:
            if not self.force_auto:
                props.unwrap_mode = 'NONE'


        if props.unwrap_mode == 'LIVE_UV':
            C.tool_settings.use_edge_path_live_unwrap = True
        else:
            C.tool_settings.use_edge_path_live_unwrap = False

        seamed = []
        unseam = []
        if self.toggle:
            init_edge = []
            for edge in mesh.edges:
                if edge.select and edge.seam:
                    edge.select = False
                    edge.seam = False
                    init_edge.append(edge)
                elif edge.select and not edge.seam:
                    edge.select = True
                    init_edge.append(edge)



        # return {'FINISHED'}
        bpy.ops.mesh.mark_seam(clear=False)

        if props.unwrap_mode == 'SELECTED':
            bpy.ops.mesh.select_linked(delimit={'SEAM'})
            bpy.ops.uv.unwrap(method= props.u_method,
                            fill_holes= props.fl_hol,
                            correct_aspect= props.cr_aspect,
                            use_subsurf_data= props.us_subf,
                            margin= props.mrgn)

        bpy.ops.mesh.select_all(action='DESELECT')

        for e in init_edge:
            e.select = True
        
        bmesh.update_edit_mesh(mesh= ms)

        return {'FINISHED'}
    def draw(self, context):
        
        scene = context.scene
        C = bpy.context
        props = scene.UVsProps
        layout = self.layout

        row = layout.row()
        row.prop(self, "toggle")

        row = layout.row(align= True)
        if self.face_count >= props.max_tris:
            row.prop(self, "force_auto")
            row.label(text = f"Faces Count: {self.face_count}")
            row = layout.row(align= True)
            row.enabled =self.force_auto
        row.prop(props, "unwrap_mode", expand = True)

class EaseamClearSeamOperator(bpy.types.Operator):
    '''Clear seam'''
    bl_idname = "mesh.easeamclearseam"
    bl_label = "Easeam Clear Seam"
    bl_options = {'REGISTER', 'UNDO'}

    all: bpy.props.BoolProperty(name= 'All Visible',
                                default= False,
                                options= {'SKIP_SAVE'})
    def execute(self, context):
        if self.all:
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.mark_seam(clear=True)
            bpy.ops.mesh.select_all(action='DESELECT')
        else:
            bpy.ops.mesh.mark_seam(clear=True)
        return {'FINISHED'}


class CylinderCutOperator(bpy.types.Operator):
    '''Auto add seam on cylender geometry
*Geaometry should be clean (Quad Based)'''
    bl_idname = "mesh.cylindercut"
    bl_label = "Tube Seam"
    bl_options = {'REGISTER', 'UNDO'}

    step: bpy.props.IntProperty(name = 'Step',
                                min= 0,
                                description= "Grow selection, similar to Select More/Less",
                                default= 0,
                                options= {'SKIP_SAVE'})
    
    force_auto: bpy.props.BoolProperty(name = 'Force Unwrap', # type: ignore
                          description= '''If this option showed up, it means the object tris exceeds the max Tris setting.
For performance reasons it's recommended to disable this option.
you can adjust the max Tris setting in Easeam preferences or enable this option to force unwrap anyway.''',
                          default = False,
                            options={'SKIP_SAVE'})
    
    def execute(self, context):
        C= bpy.context
        scene = C.scene
        props = scene.UVsProps

        # for obj in C.selected_objects:
        #     if obj != C.view_layer.objects.active:
        #         mesh = bmesh.from_edit_mesh(obj.data)
        #         for e in mesh.edges:
        #             e.select= False
        #             bmesh.update_edit_mesh(mesh= obj.data)

        self.face_count = len(C.object.data.polygons)
        if self.face_count >= props.max_tris:
            if not self.force_auto:
                props.unwrap_mode = 'NONE'


        if props.unwrap_mode == 'LIVE_UV':
            C.tool_settings.use_edge_path_live_unwrap = True
        else:
            C.tool_settings.use_edge_path_live_unwrap = False

        ms = C.active_object.data
        mesh = bmesh.from_edit_mesh(ms)

        gete = [e for e in mesh.edges if e.select == True]
        for e in gete:
            for loop in e.link_loops:
                if len(loop.vert.link_edges) == 4:
                    e.select = True
                    for step in range(self.step):
                        if len(loop.vert.link_edges) == 4:
                            loop = loop.link_loop_prev.link_loop_radial_prev.link_loop_prev
                            loop.edge.select = True
        gete2 = [e for e in mesh.edges if e.select == True]

        bpy.ops.mesh.loop_multi_select(ring=True)        
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=True, type='FACE')

        # mesh.select_mode= {'FACE'}
        # mesh.select_flush(True)
        # bpy.ops.mesh.mark_seam(clear=True)

        getf = [f for f in mesh.faces if f.select == True]

        bpy.ops.mesh.region_to_loop()

        
        bpy.ops.mesh.mark_seam(clear= False)


        gete3 = [e for e in mesh.edges if e.select == True]
        for e in gete2:
            e.select= True
        for e in gete3:
            e.select= True
        bpy.ops.mesh.mark_seam(clear=False)
        for f in getf:
            f.select= True
        # return {'FINISHED'}

        if props.unwrap_mode == 'SELECTED':
            bpy.ops.uv.unwrap(method= props.u_method,
                            fill_holes= props.fl_hol,
                            correct_aspect= props.cr_aspect,
                            use_subsurf_data= props.us_subf,
                            margin= props.mrgn)
            
        bmesh.update_edit_mesh(mesh= ms)


        return {'FINISHED'}

    def draw(self, context):
        
        scene = context.scene
        C = bpy.context
        props = scene.UVsProps
        layout = self.layout

        row = layout.row()
        row.prop(self, "step")

        row = layout.row(align= True)
        if self.face_count >= props.max_tris:
            row.prop(self, "force_auto")
            row.label(text = f"Faces Count: {self.face_count}")
            row = layout.row(align= True)
            row.enabled =self.force_auto
        row.prop(props, "unwrap_mode", expand = True)


class HideIslandOperator(bpy.types.Operator):
    '''Hide faces based linked island '''
    bl_idname = "mesh.hideisland"
    bl_label = "HideIsland"
    bl_options = {'REGISTER', 'UNDO'}

    hide_faces: bpy.props.BoolProperty(name= "Faces Only",
                                default= False,
                                description= "Show edges (Hide faces only)")

    keep_seam : bpy.props.BoolProperty(name= "Keep Edge Seam",
                                default= True,)

    def execute(self, context):
        C= bpy.context
        scene = C.scene
        easeam_prop = scene.UVsProps
        ms = C.active_object.data
        mesh = bmesh.from_edit_mesh(ms)

        bpy.ops.mesh.select_linked(delimit={'SEAM'})
        
        getf = [f for f in mesh.faces if f.select == True]

        for f in getf:
            f.hide= True
            if not self.hide_faces:
                if not self.keep_seam:
                    for e in f.edges:
                        e.hide= True
                else:
                    for e in f.edges:
                        if not e.seam:
                            e.hide= True


        bmesh.update_edit_mesh(mesh= ms)
        
        return {'FINISHED'}

    def draw(self, context):
        scene = context.scene
        layout = self.layout

        col= layout.column()
        col.prop(self, "keep_seam", icon= 'MATSPHERE')
        col.prop(self, "hide_faces", icon= 'SHADING_WIRE')


class SharpSeamOperator(bpy.types.Operator):
    '''Marking seam by marked sharp edge and/or by angle'''
    bl_idname = "mesh.sharpseam"
    bl_label = "Seam by Sharp Edge"
    bl_options = {'REGISTER', 'UNDO'}


    auto_uv_sharp: bpy.props.BoolProperty(name= 'Auto Unwrap',description="Auto unwrap after marking seams (Selected)", default= True)
    marked: bpy.props.BoolProperty(name= 'Marked',description="By edges has marked sharp", default= True)
    angle: bpy.props.BoolProperty(name= 'Angle',description="By angle setting", default= False)

    set_angle: bpy.props.FloatProperty(name= 'Min Angle',
                        description="Minimum edge angle for marking seam",
                        default= 0.523599,
                        min= 0,
                        max= 3.14159,
                        subtype = 'ANGLE')

    # m_sharpness: bpy.props.EnumProperty(
    #     name="Sharpness Method",
    #     items=(('ANGLE', "Angle", "Only sharp edge by angle setting"),
    #             ('MARKED', "Marked", "Only edges has marked sharp")),
    #     default= 'ANGLE',
    #     description="Which sharpness method will marked seam",)

    def execute(self, context):
        C= bpy.context
        scene = C.scene
        easeam_prop = scene.UVsProps
        # ms = C.active_object.data

        live_u = C.tool_settings.use_edge_path_live_unwrap
        
        C.tool_settings.use_edge_path_live_unwrap = False
        if easeam_prop.live_uv:
            C.tool_settings.use_edge_path_live_unwrap = True

        bpy.ops.mesh.select_all(action='DESELECT')
        ###
        for obj in C.selected_objects:
            ms= obj.data
            mesh = bmesh.from_edit_mesh(ms)
            bpy.ops.mesh.select_mode(type="EDGE")

            sharp_edges= [edge for edge in mesh.edges if not edge.smooth]
            sel_edges= [edge for edge in mesh.edges if not edge.select]
            
            if self.angle:
                bpy.ops.mesh.edges_select_sharp(sharpness= self.set_angle)
                bpy.ops.mesh.mark_seam(clear=False)

            if self.marked:
                for e in sharp_edges:
                    e.select= True
                    e.seam= True

            if self.auto_uv_sharp:
                bpy.ops.mesh.select_linked()
                bpy.ops.uv.unwrap(method= easeam_prop.u_method,
                                fill_holes= easeam_prop.fl_hol,
                                correct_aspect= easeam_prop.cr_aspect,
                                use_subsurf_data= easeam_prop.us_subf,
                                margin= easeam_prop.mrgn)
                bpy.ops.mesh.select_all(action='DESELECT')
            
            for e in sel_edges:
                if e.seam:
                    e.select= True
        bmesh.update_edit_mesh(mesh= ms)
        C.tool_settings.use_edge_path_live_unwrap= live_u

        return {'FINISHED'}

    def draw(self, context):
        scene = context.scene
        layout = self.layout

        col= layout.column()
        col.prop(self, "auto_uv_sharp", text= 'Unwrap', icon= 'EVENT_U')

        row= layout.row(align=True)
        row.prop(self, "marked", text= "Marked Sharp")
        row= layout.row(align=True)
        row.prop(self, "angle", text= "Use Angle")
        row= row.row(align=True)
        row.enabled= self.angle
        row.prop(self, "set_angle")

########################### UVMAP

class UVMapDeleteOperator(bpy.types.Operator):
    '''Delete UV-Map Channel'''
    bl_idname = "uv.uv_map_delete"
    bl_label = "Delete UV-Map"
    bl_options = {'REGISTER', 'UNDO'}

    batch: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'})
    
    obj_name: bpy.props.StringProperty()
    uv_name: bpy.props.StringProperty()

    def execute(self, context):
        C= bpy.context
        D = bpy.data
        if self.batch:
            objs = C.selected_objects
            objs.append(C.active_object)
            for obj in objs:
                try:
                    uv = obj.data.uv_layers[self.uv_name]
                    obj.data.uv_layers.remove(uv)
                except KeyError:
                    pass

        else:
            obj = D.objects[self.obj_name]
            uv_layer = obj.data.uv_layers[self.uv_name]
            obj.data.uv_layers.remove(uv_layer)
        
        return {'FINISHED'}
    
    def draw(self, context):
        pass

class SetActiveUVLayerOperator(bpy.types.Operator):
    '''Set Active and Render UV-Map Channel'''
    bl_idname = "uv.set_active_uv_layer"
    bl_label = "Set Active UV-Map"
    bl_options = {'REGISTER', 'UNDO'}

    batch: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'})
    obj_name: bpy.props.StringProperty()
    uv_name: bpy.props.StringProperty()

    focus: bpy.props.BoolProperty(name= 'Select objects error',
                                  description= 'Select only this object')
    
    def execute(self, context):
        C= bpy.context
        D = bpy.data

        for obj in C.selected_objects:
            if obj.type != 'MESH':
                obj.select = False

        if self.batch:
            objs = C.selected_objects
            objs.append(C.active_object)

            no_uv = []
            for obj in objs:
                try:
                    obj.data.uv_layers[self.uv_name].active = True
                    obj.data.uv_layers[self.uv_name].active_render = True
                except KeyError:
                    no_uv.append(obj.name)
            if self.focus:
                bpy.ops.object.mode_set(mode='OBJECT')
                bpy.ops.object.select_all(action='DESELECT')
                for o in no_uv:
                    obj = D.objects[o]
                    obj.select = True

            if no_uv:
                self.report({'WARNING'}, f"Objects {str(no_uv)} has no UV-Map named {self.uv_name}")


        else:
            obj = D.objects[self.obj_name]
            obj.data.uv_layers[self.uv_name].active = True
            obj.data.uv_layers[self.uv_name].active_render = True

        return {'FINISHED'}
    
    def draw(self, context):
        layout = self.layout
        col = layout.column(align= True)
        col.prop(self, 'focus')
    
class AddNewUVLayerOperator(bpy.types.Operator):
    '''Add new UV-Map Channel'''
    bl_idname = "uv.add_new_uv_layer"
    bl_label = "Add New UV Layer"
    bl_options = {'REGISTER', 'UNDO'}

    batch: bpy.props.BoolProperty(default= False,
                                options= {'SKIP_SAVE'})
    
    obj_name: bpy.props.StringProperty()
    uv_name: bpy.props.StringProperty(name= 'UV Name',
                                      default= 'UVMap')

    def execute(self, context):
        C= bpy.context
        if self.batch:
            total_uv = len(C.object.data.uv_layers)
            total_uv = total_uv + 1
            for obj in C.selected_objects:
                if obj.type == 'MESH':
                    if len(obj.data.uv_layers) < total_uv:
                            obj.data.uv_layers.new(name= self.uv_name)
                            # break
        else:
            D = bpy.data
            obj = D.objects[self.obj_name]
            obj.data.uv_layers.new(name= self.uv_name)

        return {'FINISHED'}
    
    def draw(self, context):
        layout = self.layout
        col= layout.row()
        col.prop(self, 'uv_name')

class BatchRenameUVMapOperator(bpy.types.Operator):
    """Batch Rename UV Maps of Selected Objects"""
    bl_idname = "object.batch_rename_uvmap"
    bl_label = "Batch Rename UV Maps"
    
    current_name: bpy.props.StringProperty()
    new_name: bpy.props.StringProperty(name="New UV-Map Name", default="UVMap") # type: ignore

    @classmethod
    def poll(cls, context):
        return context.selected_objects is not None and len(context.selected_objects) > 0

    def execute(self, context):
        for obj in context.selected_objects:
            if obj.type == 'MESH':
                try:
                    uv_map = obj.data.uv_layers[self.current_name]
                    uv_map.name = self.new_name
                except KeyError:
                    self.report({'WARNING'}, f"{obj.name} has no UV-Map named {self.current_name}")

        
        return {'FINISHED'}

    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)

    def draw(self, context):
        layout = self.layout
        col = layout.column(align= True)
        col.label(text= f'Selected objects with UV-Map "{self.current_name}" will be renamed')
        # col.label(text= f"Selected objects with UV-Map names")
        # col.label(text= f"{self.current_name} will be renamed.")

        row = layout.row()
        row.prop(self, "new_name")

class PruneUVMapsOperator(bpy.types.Operator):
    bl_idname = "object.prune_uvmaps"
    bl_label = "Prune UVMaps channel"
    bl_description = "Delete all UV-Maps channel except the active one and set the same name on selected objects"
    bl_options = {'REGISTER', 'UNDO'}

    uv_name: bpy.props.StringProperty(name= 'Set Name',
                                           default= 'UVMap')

    def execute(self, context):
        selected_objects = [o for o in bpy.context.selected_objects if o.type == 'MESH']
        print('UVMap name: ' + self.uv_name)
        for obj in selected_objects:
            if obj.type == 'MESH':
                if not len(obj.data.uv_layers):
                    obj.data.uv_layers.new(name= self.uv_name)

            attempt = 0
            while len(obj.data.uv_layers) > 1:
                for uvmap in obj.data.uv_layers:
                    if not uvmap.active:
                        try:
                            obj.data.uv_layers.remove(uvmap)
                        except:
                            pass
                attempt += 1
                if attempt > 100:
                    self.report({'ERROR'}, "Shomething went wrong please report this issue.")
                    break
                        
        for obj in selected_objects:
            for uvmap in obj.data.uv_layers:
                uvmap.name = self.uv_name

        self.report({'INFO'}, f'Pruned UV-Map on {len(selected_objects)} Object(s)')
        return {'FINISHED'}
    
    # def draw(self, context):
    #     layout = self.layout
    #     layout.label(text="This will prune UV Maps. Are you sure?")
    def invoke(self, context, event):
        return context.window_manager.invoke_confirm(self, event)



class HighlightObjectOperator(bpy.types.Operator):
    '''This UV-Map channel doesn't exist in active object.\nClick to select object and fix (opsional)'''
    bl_idname = "object.highlightobject"
    bl_label = "Error found"
    bl_options = {'REGISTER', 'UNDO'}

    obj_name: bpy.props.StringProperty(options= {'SKIP_SAVE'})
    uv_name: bpy.props.StringProperty(options= {'SKIP_SAVE'})
    current_uv: bpy.props.StringProperty(options= {'SKIP_SAVE'})
    fix: bpy.props.BoolProperty(name= 'Fix',
                                description= 'Rename UV-Map or delete if total UV-Maps more than in active object',
                                default= True,
                                options= {'SKIP_SAVE'}) # type: ignore
    focus: bpy.props.BoolProperty(name= 'Select Object',
                                  description= 'Select only this object') # type: ignore

    def execute(self, context):
        obj = bpy.data.objects[self.obj_name]

        if self.focus:
            bpy.ops.object.mode_set(mode='OBJECT')
            bpy.ops.object.select_all(action='DESELECT')
            obj.select = True
            # bpy.ops.view3d.view_selected(use_all_regions=False)

        if self.fix:
            uv_map = obj.data.uv_layers[self.current_uv]
            uv_map.name = self.uv_name
            if not self.uv_name:
                obj.data.uv_layers.remove(uv_map)

        return {'FINISHED'}
    
    def draw(self, context):
        layout = self.layout
        col = layout.column(align= True)
        col.prop(self, 'focus')
        col.prop(self, 'fix')


class OrderUVMapOperator(bpy.types.Operator):
    '''UV-Map order doesn't same as active object\nClick to fix'''
    bl_idname = "object.orderuvmap"
    bl_label = "Fix order UV-Map"
    bl_options = {'REGISTER', 'UNDO'}


    obj_name: bpy.props.StringProperty()

    def execute(self, context):
        C = bpy.context
        D = bpy.data
        obj= D.objects[self.obj_name]
        uv_list = C.active_object.data.uv_layers
        uv_ob = obj.data.uv_layers

        idx= 0
        tmp = '#temp-order'
        for uv,uv_a in zip(uv_ob,uv_list):
            # if uv.name != uv_a.name:
            uv_ob.active = uv_ob[uv_a.name]
            suffix = f'{tmp}{idx}'
            uv_ob.new(name= uv_a.name + suffix)

            
            idx += 1
        for uv in uv_ob:
            while tmp not in uv.name:
                try:
                    uv_ob.remove(uv)
                except:
                    break
        for uv in uv_ob:
            if tmp in uv.name:
                uv.name = uv.name.split("#temp-order")[0]
            
        return {'FINISHED'}
    def draw(self, context):
        pass

def get_all_edges_connected_to_edge(start_edge):
    # Initialize a set to keep track of visited edges
    visited_edges = set()
    # Initialize a stack for DFS
    stack = list(start_edge.link_faces)
    
    while stack:
        face = stack.pop()
        for edge in face.edges:
            if edge not in visited_edges and edge != start_edge:
                visited_edges.add(edge)
                edge.hide = True
                # Add all faces connected to this edge to the stack
                for neighbor_face in edge.link_faces:
                    if neighbor_face not in stack:
                        stack.append(neighbor_face)
    
    return list(visited_edges)

class AutoSeamTubeOperator(bpy.types.Operator):
    bl_idname = "object.autoseamtube"
    bl_label = "AutoSeamTube"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        C = context
        obj = C.object
        
        bm = bmesh.from_edit_mesh(obj.data)
        
        edge_lengths = [edge.calc_length() for edge in bm.edges]
        
        if len(edge_lengths) == 0:
            return [], []
        
        median_length = statistics.median(edge_lengths)
        
        short_edges = [edge for edge in bm.edges if edge.calc_length() < median_length]
        long_edges = [edge for edge in bm.edges if edge.calc_length() >= median_length]
        
        # for edge in long_edges:
        #     edge.select = True  # Select shorter edges
        
        root_loop = []
        all_edges =[]
        for e in long_edges:
            if not e.hide:
                e.select = True
                root_loop.append(e)
            # else:
                visited_edges = get_all_edges_connected_to_edge(e)
                all_edges.append(visited_edges)
            
        # for e in visited_edges:
        #     e.hide = False

        visited_edges = list(itertools.chain(*all_edges))
        for e in visited_edges:
            e.select = False
            e.hide = False

        
        bmesh.update_edit_mesh(obj.data)
        bpy.ops.mesh.loop_multi_select(ring=False)
        bpy.ops.mesh.mark_seam(clear=False)

        return {'FINISHED'}
