from typing import Text
import bpy
import bmesh
from bpy.types import Panel
from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       FloatVectorProperty,
                       EnumProperty,
                       PointerProperty,)

class OCV_PT_UVTools(bpy.types.Panel):
    bl_idname = "OCV_PT_UVTools"
    bl_label = "UV Tools"
    bl_category = "Easeam"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"

    @classmethod
    def poll(cls, context):
        C = bpy.context

        scene = context.scene
        easeam_prop = scene.UVsProps

        if easeam_prop.sw_panel:
            return context.mode in {'EDIT_MESH'}

    def draw(self, context):
        C = bpy.context

        scene = context.scene
        easeam_prop = scene.UVsProps
        layout = self.layout

        row = layout.row(align= True)
        row.scale_y= 1.2
        row.prop(easeam_prop, "alws_hide", icon= 'EVENT_H', icon_only= True)
        row.operator('mesh.boundaryseam', text="Mark Around Flat")
        row.prop(easeam_prop, "u_mark_around_flat", icon= 'EVENT_U', icon_only= True)

        row = layout.row(align=True)
        row.scale_y= 1.2
        row.prop(easeam_prop, "s_loop", icon= 'EVENT_S', icon_only= True)
        row.operator('mesh.limitloop', text="Seam Edge Loop")
        row.prop(easeam_prop, "u_loop_seam", icon= 'EVENT_U', icon_only= True)

        row = layout.row(align=True)
        row.scale_y= 1.2
        row.operator('mesh.cylindercut', text="Tube Seam")
        row.prop(easeam_prop, "u_tube", icon= 'EVENT_U', icon_only= True)

        row = layout.row(align=True)
        row.scale_y= 1.2
        row.operator('mesh.sharpseam', text="Seam by Sharp Edge")
        # row.prop(easeam_prop, "auto_uv_sharp", icon= 'EVENT_U', icon_only= True)

        col = layout.column(align=True)
        row = layout.row(align=True)

        col = col.box()
        row = col.row(align= True)
        row.scale_y= 1.2
        row.operator('mesh.smartseam', text="Auto Unwrap & Seam", icon= 'SHADERFX')
        row.popover('OCV_PT_AutoUnwrap', text='', icon='DRIVER_ROTATIONAL_DIFFERENCE', icon_value=1)
        col.prop(easeam_prop, "os_unwrap", text= "Only Selected")


        col = layout.column(align=True)
        col = col.box()
        row = col.row()
        row.operator('mesh.easeamclearseam', text="Clear Seam")
        row.operator('mesh.easeammarkseam', text="Mark Seam")
        row = col.row()
        row.operator('mesh.hideisland', text="Hide Island", icon= 'HIDE_OFF')

        col = layout.column()
        col.operator('mesh.reunwrap', text="Re-Unwrap", icon= 'FILE_REFRESH')
        col.prop(easeam_prop, 'auto_pack')

        row = layout.row()
        row.prop(easeam_prop, "live_uv")

class OCV_PT_AutoUnwrap(bpy.types.Panel):
    bl_idname = "OCV_PT_AutoUnwrap"
    bl_label = "Angle Limit"
    bl_options = {"INSTANCED"}
    bl_space_type = "VIEW_3D"
    bl_region_type = "WINDOW"

    def draw(self, context):
        C = bpy.context
        scene = context.scene
        easeam_prop = scene.UVsProps
        layout = self.layout
        
        row = layout.row()
        row.scale_y= 1.2
        row.prop(easeam_prop, "angle_limit")

class OCV_PT_SharpnessAngle(bpy.types.Panel):
    bl_idname = "OCV_PT_SharpnessAngle"
    bl_label = "Sharpness"
    bl_options = {"INSTANCED"}
    bl_space_type = "VIEW_3D"
    bl_region_type = "WINDOW"


    def draw(self, context):
        C = bpy.context
        scene = context.scene
        easeam_prop = scene.UVsProps
        layout = self.layout
        
        row = layout.row()
        row.scale_y= 1.2
        row.prop(easeam_prop, "sharp_limit")

class OCV_PT_UnwrapOptions(bpy.types.Panel):
    bl_idname = "OCV_PT_UnwrapOptions"
    bl_label = "Unwrap Setting"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_parent_id = "OCV_PT_UVTools"
    bl_options = {"DEFAULT_CLOSED"}

    def draw_header(self, _):
        layout = self.layout
        layout.label(text="", icon='PREFERENCES')

    def draw(self, context):
        C = bpy.context
        scene = context.scene
        easeam_prop = scene.UVsProps
        layout = self.layout
        
        row = layout.row()
        row.prop(easeam_prop, "u_method", expand= True)
        row = layout.row(align= True)
        row.prop(easeam_prop, "fl_hol")
        row = layout.row(align= True)
        row.prop(easeam_prop, "cr_aspect")
        row = layout.row(align= True)
        row.prop(easeam_prop, "us_subf")
        row = layout.row(align= True)
        row.prop(easeam_prop, "mrgn")

###############################

def UvMapsPanel(self, context):
    C= bpy.context
    scene = context.scene
    easeam_prop = scene.UVsProps
    layout = self.layout

    active_object = C.active_object

    obj = active_object
    uv_list = []

    if hasattr(obj, 'type'):
        if obj.type != 'MESH':
            col = layout.column()
            col.label(text= 'Active object is not Mesh', icon= 'ERROR')
        
    if obj and obj.type == 'MESH':
        box = layout.box()
        col = box.column(align= True)
        
        col.label(text=f"Active Object: {obj.name}", icon= 'RADIOBUT_ON')
        col.label(text=f"Selected Objects: {len(C.selected_objects)}", icon= 'RADIOBUT_OFF')

        row = box.row()
        add = row.operator("uv.add_new_uv_layer", text="", icon= 'ADD')
        add.obj_name = obj.name
        add.batch = True

        row.operator("object.prune_uvmaps", text="Prune")

        active_uv = ''
        for i, uv_layer in enumerate(obj.data.uv_layers):
            if uv_layer == obj.data.uv_layers.active:
                active_uv = uv_layer.name
            row = box.row(align= True)

            rename = row.operator("object.batch_rename_uvmap", text="", icon= 'GREASEPENCIL')
            rename.current_name = uv_layer.name

            name = row.box()
            name.scale_y= 0.5
            name.label(text=f"{uv_layer.name}")
            row = row.row()

            if uv_layer.active_render:
                icon = 'RESTRICT_RENDER_OFF'
            else:
                icon = 'RESTRICT_RENDER_ON'

            uv = row.operator("uv.set_active_uv_layer", text="", icon= icon)
            uv.batch = True
            uv.obj_name = obj.name
            uv.uv_name = uv_layer.name
            
            del_uv = row.operator("uv.uv_map_delete", text="", icon= 'TRASH')
            del_uv.obj_name = obj.name
            del_uv.uv_name = uv_layer.name
            del_uv.batch = True

            uv_list.append(uv_layer.name)
            
    if easeam_prop.compare_from_active:
        for obj in C.selected_objects:
            if obj.type == 'MESH':
                if obj.data.uv_layers:
                    if obj.data.uv_layers.active.name != active_uv:
                        col = layout.column()
                        col.label(text= "Some objects doesn't same active state", icon= 'ERROR')
                        break

    col = layout.column()
    if easeam_prop.show_each_uvmap:
        icon = 'TRIA_DOWN'
    else:
        icon = 'TRIA_RIGHT'

    col.prop(easeam_prop, 'show_each_uvmap', icon= icon)

    if easeam_prop.show_each_uvmap:
        if not C.selected_objects:
            col.label(text= 'Please select mesh objects.', icon= 'INFO')
            return
        col.prop(easeam_prop, 'compare_from_active')
        col = col.column(align=True)
        col.enabled = easeam_prop.compare_from_active

        error = 0
        for obj in C.selected_objects:
            if obj.type == 'MESH':
                if uv_list != [x.name for x in obj.data.uv_layers]:
                    error += 1

        col.prop(easeam_prop, f'show_only_error', text= f'Show only error ({str(error)} error)')

        for obj in C.selected_objects:
            if obj and obj.type == 'MESH':
                if easeam_prop.compare_from_active:
                    if easeam_prop.show_only_error:
                        if uv_list == [x.name for x in obj.data.uv_layers]:
                            obj= None

                try:
                    if obj:
                        box = layout.box()
                        row = box.row()

                    if obj == C.active_object:
                        icon= 'RADIOBUT_ON'
                    else:
                        icon = 'RADIOBUT_OFF'
                    row.label(text=f"{obj.name}", icon= icon)
                    if easeam_prop.compare_from_active:
                        if uv_list != [x.name for x in obj.data.uv_layers]:
                            uv_com = []
                            for uv in uv_list:
                                if uv in  [x.name for x in obj.data.uv_layers]:
                                    uv_com.append(True)
                                else:
                                    uv_com.append(False)
                            if False not in uv_com and len(uv_list) == len(obj.data.uv_layers):
                                obj_uv= [x.name for x in obj.data.uv_layers]
                                for uv,uv_o in zip(uv_list,obj_uv):
                                    if uv != uv_o:
                                        order = row.operator("object.orderuvmap",text='', icon= 'ERROR')
                                        order.obj_name = obj.name
                                        break

                            row.label(icon= 'CANCEL')

                    row.operator("uv.add_new_uv_layer", text="", icon= 'ADD').obj_name = obj.name
                    
                    if easeam_prop.compare_from_active:
                        for u in uv_list:
                            uv_name= [x.name for x in obj.data.uv_layers]
                            if u not in uv_name:
                                row = box.box()
                                row = row.row()
                                row.label(text=f"Missing: {u}")
                                
                                add = row.operator("uv.add_new_uv_layer", text="", icon= 'ADD')
                                add.obj_name = obj.name
                                add.uv_name = u


                    for i, uv_layer in enumerate(obj.data.uv_layers):
                        row = box.row()
                        if not uv_layer.active:
                            row.label(icon= 'LAYER_USED')
                        else:
                            row.label(icon= 'LAYER_ACTIVE')
                        row.prop(uv_layer, "name", text=f"")
                        
                        if easeam_prop.compare_from_active:
                            if uv_layer.name not in uv_list:
                                objh = row.operator("object.highlightobject", text="", icon= 'ERROR')
                                objh.obj_name = obj.name
                                objh.current_uv = uv_layer.name
                                for u in uv_list:
                                    uv_name= [x.name for x in obj.data.uv_layers]
                                    if u not in uv_name:
                                        objh.uv_name = u
                                        # break


                        if uv_layer.active_render:
                            icon = 'RESTRICT_RENDER_OFF'
                        else:
                            icon = 'RESTRICT_RENDER_ON'
                        uv = row.operator("uv.set_active_uv_layer", text="", icon= icon)
                        uv.obj_name = obj.name
                        uv.uv_name = uv_layer.name
                        
                        del_uv = row.operator("uv.uv_map_delete", text="", icon= 'TRASH')
                        del_uv.obj_name = obj.name
                        del_uv.uv_name = uv_layer.name


                    if not len(obj.data.uv_layers):
                        row = box.row()
                        row.label(text=f"No UV-Map", icon= 'ERROR')
                except:
                    pass




class UVLoopLayersPanel(bpy.types.Panel):
    bl_label = "UV Maps"
    bl_idname = "OCV_PT_UVMaps"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Easeam"

    def draw(self, context):
        UvMapsPanel(self, context)

#UV-Map

class EaseamUVPanel(bpy.types.Panel):
    bl_label = "Easeam"
    bl_idname = "UV_PT_simple_panel"
    bl_space_type = 'IMAGE_EDITOR'
    bl_region_type = "UI"
    bl_category = "Easeam"


    def draw(self, context):
        UvMapsPanel(self, context)

