bl_info = {
    "name" : "Easeam",
    "author" : "Mohtar-Orcavia",
    "description" : "An easy and simple tools for marking seam (UV)",
    "blender" : (4, 0, 1),
    "version" : (1, 1, 0),
    "location" : "View3D > Easeam (N panel) Or Pie Menu",
    "warning" : "",
    "category" : "UV",
    "tracker_url": "",
}

import bpy



from . props import UVsProps
from . OP_UVs import BoundarySeamOperator
from . OP_UVs import ReUnwrapOperator
from . OP_UVs import LimitLoopOperator
from . OP_UVs import SmartSeamOperator
from . OP_UVs import EaseamMarkSeamOperator
from . OP_UVs import EaseamClearSeamOperator
from . OP_UVs import CylinderCutOperator
from . OP_UVs import HideIslandOperator
# from . OP_UVs import MultiEditBoundaryOperator
from . OP_UVs import SharpSeamOperator

from . OP_UVs import UVMapDeleteOperator
from . OP_UVs import SetActiveUVLayerOperator
from . OP_UVs import AddNewUVLayerOperator
from . OP_UVs import BatchRenameUVMapOperator
from . OP_UVs import PruneUVMapsOperator
from . OP_UVs import HighlightObjectOperator
from . OP_UVs import OrderUVMapOperator
from . OP_UVs import AutoSeamTubeOperator

from . Pie_Easeam import VIEW3D_MT_PIE_Easeam
from . Pie_Easeam import PIE_MT_UnwrapOptions

from . UI_Easeam import OCV_PT_UVTools 
from . UI_Easeam import OCV_PT_AutoUnwrap 
from . UI_Easeam import OCV_PT_SharpnessAngle 
from . UI_Easeam import OCV_PT_UnwrapOptions 
from . UI_Easeam import UVLoopLayersPanel 
from . UI_Easeam import EaseamUVPanel 

from bpy.props import (
                       PointerProperty
                       )
from .translation.translate import GetTranslationDict

#############################PANEL
def get_hotkey_entry_item(km, kmi_name, kmi_value, properties):
    for i, km_item in enumerate(km.keymap_items):
        if km.keymap_items.keys()[i] == kmi_name:
            if properties == 'name':
                if km.keymap_items[i].properties.name == kmi_value:
                    return km_item
            elif properties == 'tab':
                if km.keymap_items[i].properties.tab == kmi_value:
                    return km_item
            elif properties == 'none':
                return km_item
    return None

class OCV_PT_EaseamPreference(bpy.types.AddonPreferences):
    bl_idname = __package__

    def draw(preference, context):
        layout= preference.layout
        C = bpy.context
        scene = C.scene
        easeam_prop = scene.UVsProps
        
        wm = bpy.context.window_manager
        kc = wm.keyconfigs.user

        km = kc.keymaps['Mesh']
        kmi = get_hotkey_entry_item(km, 'wm.call_menu_pie', 'VIEW3D_MT_PIE_Easeam', 'name')

        box = layout.box()
        row= box.row()
        row.label(text= "Easeam Keymap (Pie Menu):")
        row.prop(kmi, "type", text="", full_event=True)

        row= box.row()
        row.prop(easeam_prop, "sw_panel")
        row= box.row()
        row.label(text= "Max Faces to disable auto unwrap:")
        row.prop(easeam_prop, "max_tris", text='')

#############################PANEL-END
classes = (UVsProps,
            OCV_PT_UVTools,
            OCV_PT_AutoUnwrap,
            OCV_PT_UnwrapOptions,
            EaseamUVPanel,
            VIEW3D_MT_PIE_Easeam,
            BoundarySeamOperator,
            ReUnwrapOperator,
            LimitLoopOperator,
            SmartSeamOperator,
            OCV_PT_EaseamPreference,
            PIE_MT_UnwrapOptions,
            EaseamMarkSeamOperator,
            EaseamClearSeamOperator,
            CylinderCutOperator,
            HideIslandOperator,
            # MultiEditBoundaryOperator,
            SharpSeamOperator,
            OCV_PT_SharpnessAngle,
            UVLoopLayersPanel,
            UVMapDeleteOperator,
            SetActiveUVLayerOperator,
            AddNewUVLayerOperator,
            BatchRenameUVMapOperator,
            PruneUVMapsOperator,
            HighlightObjectOperator,
            OrderUVMapOperator,
            AutoSeamTubeOperator
            )

addon_keymaps = []

def register():
    try:
        for cls in classes:
            bpy.utils.register_class(cls)
        bpy.types.Scene.UVsProps = PointerProperty(type=UVsProps)
        wm = bpy.context.window_manager
        if wm.keyconfigs.addon:
            km = wm.keyconfigs.addon.keymaps.new(name='Mesh')
            kmi = km.keymap_items.new('wm.call_menu_pie', 'S', 'PRESS', alt=True)
            kmi.properties.name = "VIEW3D_MT_PIE_Easeam"
            addon_keymaps.append((km, kmi))
    except:
        pass
    bpy.app.translations.register(__package__, GetTranslationDict())
    from pathlib import Path
    print(f"{Path(__file__).parent.name} 插件由xz-blender汉化!感谢支持!")

def unregister():
    try:
        del bpy.types.Scene.UVsProps
        for cls in reversed(classes):
            bpy.utils.unregister_class(cls)
    except:
        pass
        
    bpy.app.translations.unregister(__package__)

if __name__ == "__main__":
    register()

