from os import name
from typing import Text
import bpy
from bpy.props import (StringProperty,
                       BoolProperty,
                       IntProperty,
                       FloatProperty,
                       FloatVectorProperty,
                       EnumProperty,
                       PointerProperty,
                       )

def UpdatedFunction(self, context):
    C = bpy.context
    props = C.scene.UVsProps
    
    if props.live_uv:
        C.scene.tool_settings.use_edge_path_live_unwrap = True
    else:
        C.scene.tool_settings.use_edge_path_live_unwrap = False

# def update_unwrap_mode(self, context):
#     print('update_unwrap_mode')
#     C = bpy.context
#     props = C.scene.UVsProps
    
#     if props.unwrap_mode == 'SELECTED':
#         props.u_mark_around_flat = True

#     elif props.unwrap_mode == 'LIVE_UV':
#         props.live_uv = True
    
#     else:
#         props.live_uv = False
#         props.u_mark_around_flat = False

class UVsProps(bpy.types.PropertyGroup):

    live_uv: BoolProperty(name= 'Use Live Unwrap',description="Auto unwrap after marking seams (whole Mesh).\n\"WARNING\" Might be slow on high amount faces", default= False,
                            update=UpdatedFunction)
    u_mark_around_flat: BoolProperty(name= 'Auto Unwrap',description="Auto unwrap after marking seams (Selected)", default= True)
    no_ask: BoolProperty(name= 'Don\'t ask again',description="Skip this dialog on future action", default= False)
    s_loop: BoolProperty(name= 'Mark Seam',description="Mark edge seam directly\nDisable to select only", default= True)

    #Auto Unwrap
    angle_limit: FloatProperty(name= 'Angle Limit',
                        description="Lower for more projection groups, higher for less distortion",
                        default= 1.15192,
                        min= 0,
                        max= 1.55334,
                        subtype = 'ANGLE')
    os_unwrap: BoolProperty(name= 'Only Selected',description="Only selected will affect", default= True)
    auto_pack: BoolProperty(name= 'Auto Pack UV Island',
                            description="Perform Auto-pack UV Island for optimize UV Space",
                            default= False,
                            options={'SKIP_SAVE'})

    #Unwrap method
    u_method: EnumProperty(
                        items= [('ANGLE_BASED', 'Angle Based', ""),
                        ('CONFORMAL', 'Conformal', "")],
                        name="Method",
                        description="Unwrapping method (Angle Based usually gives better results than Conformal, while being somewhat slower)",
                        default='CONFORMAL',
                        options={'ANIMATABLE'}
                        )

    fl_hol: BoolProperty(name= 'Fill Holes',description="Virtual fill holes in mesh before unwrapping, to better avoid overlaps and preserve symmetry", default= True)
    cr_aspect: BoolProperty(name= 'Correct Aspect',description="Map UVs taking image aspect ratio into account", default= True)
    us_subf: BoolProperty(name= 'Use Subdivision Surface',description="Map UVs taking vertex position after Subdivision Surface modifier has been applied", default= False)
    alws_hide: BoolProperty(name= 'Directly hide',description="Directly hide selected", default= False)

    mrgn: FloatProperty(name= 'Margin',
                        description="Space between islands",
                        default= 0.001,
                        min= 0,
                        max= 1,
                        subtype = 'FACTOR')

    #Limit Loop
    u_loop_seam: BoolProperty(name= 'Auto Unwrap',description="Auto unwrap after marking seams (Selected)", default= False)
    #Tube seam
    u_tube: BoolProperty(name= 'Auto Unwrap',description="Auto unwrap after marking seams (Selected)", default= True)

    # Preference
    sw_panel: BoolProperty(name= 'Show Panel In 3D View',description="Show Easeam panel in 3D View (Edit Mesh > N panel)", default= True)

    #Seam sharp
    auto_uv_sharp: BoolProperty(name= 'Auto Unwrap',description="Auto unwrap after marking seams (Selected)", default= True)
    sharp_limit: FloatProperty(name= 'Min Angle',
                        description="Minimum edge angle for marking seam",
                        default= 0.785398,
                        min= 0,
                        max= 3.14159,
                        subtype = 'ANGLE')

    m_sharpness: EnumProperty(
        name="Sharpness Method",
        items=(('ANGLE', "Angle", "Only sharp edge by angle setting"),
                ('MARKED', "Marked", "Only edges has marked sharp")),
        default= 'ANGLE',
        description="Which sharpness method will marked seam",)

    marked: BoolProperty(name= 'Marked',description="By edges has marked sharp", default= True)
    angle: BoolProperty(name= 'Angle',description="By angle setting", default= True)

    show_each_uvmap : BoolProperty(name= 'Show each',description="Show UV Maps each selected object", default= False)
    compare_from_active : BoolProperty(name= 'Compare from active object',description="Compare UV-Maps from active object", default= True)
    show_only_error : BoolProperty(name= 'Show only error',
                                   description="Panel UV-Maps will be hiden if object already has same UV-Maps with active object", default= False)
    unwrap_mode: EnumProperty(
        name="Unwrap Mode",
        items=(('SELECTED', "Selected", "Only selected Faces", 'EVENT_U', 0),
                ('LIVE_UV', "Live Unwrap", "Using Live Unwrap (Unwrap whole Mesh)", 'EVENT_L', 1),
                ('NONE', "None", "Nothing unwrap only marking seam", 2)),
        default= 'NONE',
        # update = update_unwrap_mode,
        description="Select Unwrap Mode, Using Live Unwrap may cause slow on high amount Faces",)
    
    max_tris: IntProperty(name = 'Max Faces for auto unwrap',
                          description= 'Max Faces to disable auto unwrap',
                          default = 20000)
    force_auto: BoolProperty(name = 'Force Unwrap',
                          description= '''If this option showed up, it means the object Faces exceeds the max Faces setting.
For performance reasons it's recommended to disable this option.
you can adjust the max Tris setting in Easeam preferences or enable this option to force unwrap anyway.''',
                          default = False,
                            options={'SKIP_SAVE'})